<?php
/**
 * @file
 * Common functions available to user_generated_content module.
 */

/**
 * Validate form value is a node type
 *
 * @param string $node_type
 *   Form value from project node.
 *
 * @return boolean
 *   Valid node type.
 */
function user_generated_content_is_valid_node_type($node_type) {
  $types = node_type_get_types();
  return array_key_exists($node_type, $types);
}

/**
 * Get field definitions to build UGC form.
 *
 * @param string $node_type
 *   Node type to use for form
 * @param int $project_nid
 *   The node id for the UGC project node
 * @param array $state
 *   Form state
 *
 * @return array
 *   Definition of form fields
 */
function user_generated_content_get_form_field_definition($node_type, $project_nid, $state) {
  // Get node type definition.
  $result = db_query('SELECT * FROM {node_type} WHERE type = :type LIMIT 1', array(':type' => $node_type));
  $type = $result->fetchObject();
  // Load the list of fields relating to this node type.
  $fields = field_info_instances('node', $node_type);
  // Supplement the array of field instances with each field's configuration info.
  foreach ($fields as $id => &$f) {
    $f['config'] = field_info_field($id);
  }
  $extra_weights = variable_get('content_extra_weights_' . $node_type, array());
  // Build definitions for each field.
  $definition = array();
  // Title field definition.
  if ($type->has_title) {
    $params = array(
      'label' => $type->title_label,
      'cardinality' => 1,
      'required' => TRUE,
      'weight' => ((!empty($extra_weights['title'])) ? $extra_weights['title'] : 0),
    );
    $definition[] = user_generated_content_build_text_field('title', 'field', $params, $state);
  }
  // Loop defined fields for node type.
  foreach ($fields as $field) {
    // Check for ugc_extended_build_XX_field function,
    if (function_exists('ugc_extended_build_' . $field['widget']['type'] . '_field')) {
      $custom_field = call_user_func('ugc_extended_build_' . $field['widget']['type'] . '_field', $field, $project_nid, $node_type, $state);
      if (!empty($custom_field)) {
        $definition[] = $custom_field;
      }
    }
    // Check the field display format and, if it is set to hidden, create a hidden form element.
    elseif (!empty($field['display']['default']['type']) && $field['display']['default']['type'] == 'hidden') {
      $default = '';
      // Is there a default value provided for this field?
      if (!empty($field['default_value'][0]) && !empty($field['default_value'][0]['value'])) {
        $default = $field['default_value'][0]['value'];
      }
      // Create a hidden form element with the default value, if one is given.
      $definition[] = user_generated_content_build_hidden_field($field['field_name'], $default);
    }
    else {
      // Check widget type is supported by default.
      if (user_generated_content_field_supported_natively($field)) {
        // Get field settings.
        $widget_settings = $field['widget']['settings'];
        // Get field default value.
        $default = '';
        // Build field definition based on widget type.
        switch ($field['widget']['type']) {
          case 'number':
          case 'text_textfield':
            $params = array(
              'label' => $field['label'],
              'cardinality' => $field['config']['cardinality'],
              'required' => $field['required'],
              'weight' => $field['display']['default']['weight'],
              'description' => $field['description'],
              'default' => $default,
              'maxlength' => (isset($field['config']['settings']['max_length'])) ? $field['config']['settings']['max_length'] : 0,
              'size' => (isset($field['widget']['settings']['size'])) ? $field['widget']['settings']['size'] : 0,
            );
            $definition[] = user_generated_content_build_text_field($field['field_name'], 'field', $params, $state);
            break;

          case 'text_textarea':
            $params = array(
              'label' => $field['label'],
              'cardinality' => $field['config']['cardinality'],
              'required' => $field['required'],
              'weight' => $field['display']['default']['weight'],
              'description' => $field['description'],
              'default' => $default,
              'rows' => (isset($field['widget']['settings']['rows'])) ? $field['widget']['settings']['rows'] : 0,
            );
            $definition[] = user_generated_content_build_text_field($field['field_name'], 'area', $params, $state);
            break;

          case 'options_select':
            $params = array(
              'label' => $field['label'],
              'cardinality' => $field['config']['cardinality'],
              'required' => $field['required'],
              'weight' => $field['display']['default']['weight'],
              'description' => $field['description'],
              'default' => $default,
            );
            // Figure out if the field references a vocabulary.
            $options = array();
            $taxonomy_field = ($field['config']['type'] == 'taxonomy_term_reference') ? TRUE : FALSE;
            if ($taxonomy_field) {
              // Load in the terms of the taxonomy and overwrite the $options array.
              $options = taxonomy_allowed_values($field['config']);
            }
            else {
              // Load in the allowed values for this field.
              $options = user_generated_content_get_allowed_values($field['config']);
            }
            $definition[] = user_generated_content_build_options_field($field['field_name'], 'select', $options, $params, $state);
            break;

          case 'options_buttons':
            $params = array(
              'label' => $field['label'],
              'cardinality' => $field['config']['cardinality'],
              'required' => $field['required'],
              'weight' => $field['display']['default']['weight'],
              'description' => $field['description'],
              'default' => $default,
            );
            // Figure out if the field references a vocabulary.
            $options = array();
            $taxonomy_field = ($field['config']['type'] == 'taxonomy_term_reference') ? TRUE : FALSE;
            if ($taxonomy_field) {
              // Load in the terms of the taxonomy and overwrite the $options array.
              $options = taxonomy_allowed_values($field['config']);
            }
            else {
              // Load in the allowed values for this field.
              $options = user_generated_content_get_allowed_values($field['config']);
            }
            $definition[] = user_generated_content_build_options_field($field['field_name'], 'button', $options, $params, $state);
            break;

          case 'options_onoff':
            $params = array(
              'label' => $field['label'],
              'required' => $field['required'],
              'weight' => $field['display']['default']['weight'],
              'description' => $field['description'],
              'default' => $default,
            );
            // Get select options.
            $options = user_generated_content_get_allowed_values($field['config']);
            $definition[] = user_generated_content_build_checkbox_field($field['field_name'], $options, $params, $state);
            break;
        }
      }
    }
  }
  // Add requried hidden fields.
  $definition[] = user_generated_content_build_hidden_field('ugc_project_id', $project_nid);
  $definition[] = user_generated_content_build_hidden_field('ugc_form_node', $node_type);
  // Add submit button.
  $definition[] = user_generated_content_build_button_field('create_content', array('label' => 'Create content', 'weight' => 1000));
  return $definition;
}

/**
 * Build a text field definition.
 *
 * @param string $field_name
 *   Field name.
 * @param string $type
 *   Field text type, field or area.
 * @param array $params
 *   The text field parameters.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_text_field($field_name, $type, $params, $form_state) {
  $default_value = (!empty($params['default'])) ? $params['default'] : '';
  $field_type = (($type == 'area') ? 'textarea' : 'textfield');
  $weight = (!empty($params['weight'])) ? $params['weight'] : 0;
  $title = (!empty($params['label'])) ? check_plain($params['label']) : '';
  $description = (!empty($params['description'])) ? field_filter_xss($params['description']) : '';
  $cardinality = (!empty($params['cardinality'])) ? $params['cardinality'] : 1;
  $multiple = $cardinality > 1 || $cardinality == FIELD_CARDINALITY_UNLIMITED;
  $columns = (!empty($params['columns'])) ? $params['columns'] : '';
  $id_prefix = 'ugc-' . $field_name;
  $wrapper_id = drupal_html_id($id_prefix . '-add-more-wrapper');
  $langcode = LANGUAGE_NONE;
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($field_name, $field_type);
  // If this form is being rebuilt, there's a chance we're being asked to add
  // another input to one of the fields. Check the form state to see if this is
  // the case.
  $add_field_name = (!empty($form_state['add_field']) ? $form_state['add_field'] : '');
  // Determine the number of widgets to display.
  switch ($cardinality) {
    case FIELD_CARDINALITY_UNLIMITED:
      // Work out number of inputs to show.
      $max = 1;
      if (!empty($form_state['values'][$field_name][$langcode])) {
        // Count the number of child values with numeric keys.
        $max = 0;
        foreach ($form_state['values'][$field_name][$langcode] as $key => $value) {
          if (is_numeric($key)) {
            $max++;
          }
        }
        // See if this is the field we should be adding one to.
        if ($add_field_name == $field_name) {
          $max++;
        }
      }
      break;

    default:
      $max = $cardinality;
      break;
  }
  // Add the child form elements.
  $field_elements = array();
  for ($delta = 0; $delta < $max; $delta++) {
    $field_elements[$delta] = array(
      '#type' => $field_type,
      '#field_name' => $field_name,
      '#language' => $langcode,
      '#title' => $title,
      '#description' => $description,
      // Only the first widget should be required.
      '#required' => ($delta == 0 && $params['required']),
      '#delta' => $delta,
      '#weight' => $weight + ($delta / 1000000),
      '#default_value' => $default_value,
    );
    if (!empty($theme)) {
      $field_elements[$delta]['#theme'] = $theme;
    }
    if (is_array($columns)) {
      $field_elements[$delta]['#columns'] = array_keys($params['columns']);
    }
  }
  // Create a parent element and ensure that the sub element(s) are nested
  // correctly within it. Multiple elements should be offset at numeric indexes,
  // single elements should just form part of the parent field definition.
  $field_element = array();
  if (count($field_elements) == 1 && $cardinality != FIELD_CARDINALITY_UNLIMITED) {
    $field_element = $field_elements[0];
  }
  else {
    $field_element = $field_elements;
  }
  // Extend the base form element.
  if ($field_element) {
    $field_element += array(
      '#field_name' => $field_name,
      '#cardinality' => $cardinality,
      '#title' => $title,
      '#description' => $description,
      '#max_delta' => $max,
    );
    // Add a theme, if we have one for this field type.
    if (!empty($theme)) {
      $field_element['#theme'] = $theme;
    }
    // Add 'add more' button, if not working with a programmed form.
    if ($cardinality == FIELD_CARDINALITY_UNLIMITED && empty($form_state['programmed'])) {
      $field_element['add_more'] = array(
        '#type' => 'submit',
        '#name' => strtr($id_prefix, '-', '_') . '_add_more',
        '#value' => t('Add another item'),
        '#attributes' => array('class' => array('field-add-more-submit')),
        '#weight' => $weight + 0.999999,
        '#ajax' => array(
          'callback' => 'user_generated_content_ajax_add_another',
          'wrapper' => $wrapper_id,
          'effect' => 'fade',
        ),
      );
    }
  }
  // Also aid in theming of field widgets by rendering a classified container.
  $addition[$field_name] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array(
        'field-type-' . drupal_html_class($type),
        'field-name-' . drupal_html_class($field_name),
      ),
    ),
    '#weight' => $weight,
    '#tree' => TRUE,
    // The '#language' key can be used to access the field's form element
    // when $langcode is unknown.
    '#language' => $langcode,
    $langcode => $field_element,
    '#prefix' => '<div id="' . $wrapper_id . '">',
    '#suffix' => '</div>',
  );
  // Override the default single-field container settings if this is a multiple field.
  if ($multiple) {
    $addition[$field_name]['#title'] = $title;
    $addition[$field_name]['#description'] = $description;
    $addition[$field_name]['#theme'] = 'ugc_multi_field';
    $addition[$field_name]['#required_flag'] = $params['required'];
  }
  return $addition;
}

/**
 * Build a options field definition (select/checkbox/radio button).
 *
 * @param string $name
 *   Field name.
 * @param string $type
 *   Field type  select / button.
 * @param array $options
 *   Select options, value => text.
 * @param array $params
 *   Parameters for text field.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_options_field($name, $type, $options, $params, $state) {
  // Set field type.
  $field_type = $type;
  if ($type == 'button' && !empty($params['cardinality']) && $params['cardinality'] != 1) {
    $field_type = 'checkboxes';
  }
  elseif ($type == 'button') {
    $field_type = 'radios';
  }
  // Set default value.
  $default = (!empty($params['default'])) ? $params['default'] : '';
  $default = (!empty($state['values'][$name])) ? $state['values'][$name] : $default;
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($name, $field_type);
  // Define field.
  $field = array(
    $name => array(
      '#type' => $field_type,
      '#title' => (!empty($params['label'])) ? $params['label'] : '',
      '#options' => $options,
      '#default' => $default,
      '#weight' => (!empty($params['weight'])) ? $params['weight'] : 0,
    ),
  );
  // Check to see if this should be a multi-field select box.
  if (!empty($params['cardinality']) && $params['cardinality'] != 1 && $type == 'select') {
    $field[$name]['#multiple'] = TRUE;
    $field[$name]['#size'] = 5;
  }
  // Add the field description, if given.
  if (!empty($params['description'])) {
    $field[$name]['#description'] = $params['description'];
  }
  // Add a theme, if we have one for this field type.
  if (!empty($theme)) {
    $field[$name]['#theme'] = $theme;
  }
  return $field;
}

/**
 * Build a single checkbox field definition.
 *
 * @param string $name
 *   Field name.
 * @param array $options
 *   Select options, value => text.
 * @param array $params
 *   Parameters for texy field.
 * @param array $state
 *   Form state.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_checkbox_field($name, $options, $params, $state) {
  // Set theme function.
  $theme = user_generated_content_get_field_theme_function($name, 'checkbox');
  // Get selected value.
  $keys = array_keys($options);
  $on_value = (!empty($keys) && isset($keys[1])) ? $keys[1] : NULL;
  // Set default value.
  $default = (!empty($params['default'])) ? $params['default'] == $on_value : FALSE;
  $default = (!empty($state['values'][$name])) ? $state['values'][$name] == $on_value : $default;
  // Define field.
  $field = array(
    $name => array(
      '#type' => 'checkbox',
      '#title' => (!empty($options[$on_value])) ? $options[$on_value] : $params['label'],
      '#default_value' => $default,
      '#weight' => (!empty($params['weight'])) ? $params['weight'] : 0,
      '#return_value' => $on_value,
    ),
  );
  // Add the field description, if given.
  if (!empty($params['description'])) {
    $field[$name]['#description'] = $params['description'];
  }
  // Add a theme, if we have one for this field type.
  if (!empty($theme)) {
    $field[$name]['#theme'] = $theme;
  }
  return $field;
}

/**
 * Build a text field definition.
 *
 * @param string $name
 *   Field name.
 * @param string $default
 *   Default value.
 *
 * @return array
 *   Definition for field.
 */
function user_generated_content_build_hidden_field($name, $default) {
  $field = array(
    $name => array(
      '#type' => 'hidden',
      '#default_value' => $default,
      '#weight' => 1000,
    ),
  );
  return $field;
}

/**
 * Build a button field definition.
 *
 * @param string $name
 *   Field name.
 * @param array $params
 *   Extra parameters
 *   format - array(
 *     'label' - button label
 *     'weight' - field ordering weight
 *     'validate' - define a validation function
 *     'submit' - define a submit function
 *     'ajax' - define an AJAX callback
 *   ).
 *
 * @return array
 *   Definition of field
 */
function user_generated_content_build_button_field($name, $params) {
  $field = array(
    $name => array(
      '#type' => 'submit',
      '#value' => $params['label'],
    ),
  );
  // Add weight parameter.
  if (isset($params['weight'])) {
    $field[$name]['#weight'] = $params['weight'];
  }
  // Add validate function.
  if (!empty($params['validate'])) {
    $field[$name]['#validate'] = array($params['validate']);
  }
  // Add submit function.
  if (!empty($params['submit'])) {
    $field[$name]['#submit'] = array($params['submit']);
  }
  // Add AJAX element.
  if (!empty($params['ajax'])) {
    $field[$name]['#ajax'] = $params['ajax'];
  }
  return $field;
}

/**
 * Get theme function name to apply to a field.
 *
 * @param string $name
 *   Field name.
 * @param string $type
 *   Form api field type.
 *
 * @return string
 *   Function name, NULL if no theme function exists.
 */
function user_generated_content_get_field_theme_function($name, $type) {
  $functions = array('ugc_field_' . $name, 'ugc_field_' . $type);
  foreach ($functions as $function) {
    if (user_generated_content_theme_function_exists($function)) {
      return $function;
    }
  }
  return NULL;
}

/**
 * Check theme function exists.
 *
 * $param string $function
 *   Name of function to check.
 *
 * @return boolean
 *   Function exists
 */
function user_generated_content_theme_function_exists($function) {
  global $theme, $theme_engine;
  if (empty($theme)) {
    return FALSE;
  }
  if (function_exists($theme . '_' . $function)) {
    return TRUE;
  }
  elseif (!empty($theme_engine) && function_exists($theme_engine . '_' . $function)) {
    return TRUE;
  }
  elseif (function_exists('theme_' . $function)) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Get allowed values for field.
 *
 * @param string $field
 *   The field global config array from Drupal.
 *
 * @return array
 *   Options.
 */
function user_generated_content_get_allowed_values($field) {
  $options = list_allowed_values($field);
  return $options;
}
